// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:wyatt_authentication_bloc/src/features/sign_in/sign_in.dart';
import 'package:wyatt_form_bloc/wyatt_form_bloc.dart';

/// {@template sign_in_listener}
/// Widget that listens and builds a child based on the state of
/// the sign in cubit
/// {@endtemplate}
class SignInListener<Data> extends StatelessWidget {
  /// {@macro sign_in_listener}
  const SignInListener({
    required this.child,
    this.onProgress,
    this.onSuccess,
    this.onError,
    this.customBuilder,
    super.key,
  });

  /// Callback to show when the sign in is in progress
  final void Function(BuildContext context)? onProgress;

  /// Callback to show when the sign in is successful
  final void Function(BuildContext context)? onSuccess;

  /// Callback to show when the sign in is unsuccessful
  final void Function(
    BuildContext context,
    FormStatus status,
    String? errorMessage,
  )? onError;

  /// Custom builder to show when the sign in is in progress, successful, or
  /// unsuccessful
  final void Function(BuildContext context, SignInState state)? customBuilder;

  /// Child of the widget
  final Widget child;

  @override
  Widget build(BuildContext context) =>
      BlocListener<SignInCubit<Data>, SignInState>(
        listener: (context, state) {
          if (customBuilder != null) {
            return customBuilder!(context, state);
          }

          if (onSuccess != null &&
              state.status == FormStatus.submissionSuccess) {
            return onSuccess!(context);
          }
          if (onProgress != null &&
              state.status == FormStatus.submissionInProgress) {
            return onProgress!(context);
          }
          if (onError != null &&
              (state.status == FormStatus.submissionCanceled ||
                  state.status == FormStatus.submissionFailure)) {
            return onError!(context, state.status, state.errorMessage);
          }
        },
        child: child,
      );
}
